<?php
/**
 * Compras\Logic\SolicitacaoCompras
 *
 * @category Php
 * @package  Compras\Logic\SolicitacaoCompras
 * @author   Teknisa <tecfood@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
namespace Compras\Logic\SolicitacaoCompras;

use \Geral\Util\DateUtil;
use \Setforms2\Runtime\Helper\Procedure;
use \Setforms2\Runtime\Helper\Procedure\Param;
use \Suprimentos\Repositories\Movopsol;
use \Geral\Entities\Solicita;
use \Geral\Entities\Itemsoli;

/**
 * Class SolicitacaoCompras
 *
 * @category Php
 * @package  Compras\Logic\SolicitacaoCompras
 * @author   Teknisa <tecfood@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
class SolicitacaoCompras
{
    /**
     * Executa procedure GERA_SOLICITACAO
     *
     * @param String $nrorg       Nrorg
     * @param String $filial      Filial
     * @param String $codigo_nece Codigo_nece
     * @param String $operador    Operador
     * @param String $extra       Extra
     *
     * @throws \Exception
     *
     * @return null
     */
    public function executaProcedureGeraSolicitacao(
        $nrorg, $filial, $codigo_nece, $operador, $extra
    )
    {
        try {
            $procedure
                = new Procedure("GERA_SOLICITACAO"); //Procedure 'SF_GERA_SOLICITACAO'
            self::addParametrosGeraSolicitacao(
                $procedure, $nrorg, $filial, $codigo_nece, $operador, $extra
            );
            $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Adiciona os parametros da procedure GERA_SOLICITACAO.
     *
     * @param Procedure $procedure   Setforms2\Runtime\Helper\Procedure $procedure
     * @param String    $nrorg       Nrorg
     * @param String    $filial      Filial
     * @param String    $codigo_nece Codigo_nece
     * @param String    $operador    Operador
     * @param String    $extra       Extra
     *
     * @return null
     */
    protected static function addParametrosGeraSolicitacao(
        Procedure $procedure, $nrorg, $filial, $codigo_nece, $operador, $extra
    )
    {
        /*$procedure->addParam(
            new Param(
                'P_NRORG',
                Param::PARAM_INPUT,
                $nrorg,
                Param::PARAM_TYPE_INT
            )
        );*/
        $procedure->addParam(
            new Param(
                'P_CDFILILANC',
                Param::PARAM_INPUT,
                $filial,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_NRNECECOMP',
                Param::PARAM_INPUT,
                $codigo_nece,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_CDOPERLANC',
                Param::PARAM_INPUT,
                $operador,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_EXTRA',
                Param::PARAM_INPUT,
                $extra,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_MSGEXTRA',
                Param::PARAM_INPUT,
                'Cálculo da Necessidade de Compras - Extra',
                Param::PARAM_TYPE_STR
            )
        );
    }

    /**
     * Salva uma lista de itens para a solicitacao de compras.
     *
     * @param array $params Params
     *
     * @return null
     * @throws \Exception
     */
    public static function gravaListaProdutos(array $params)
    {
        try {
            self::validaGravacaoListaProdutos($params);
            /**
             * Itemsoli
             *
             * @var $itemsoliRepository \Geral\Repositories\Itemsoli
             */
            $itemsoliRepository = \Geral\Entities\Itemsoli::getRepository();
            $itemsoliRepository->insereListaDeItens($params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Importa os itens de uma solicitacao anterior.
     *
     * @param array $params Params
     *
     * @return \Doctrine2\DBAL\Driver\Statement
     * @throws \Exception
     */
    public static function importaSolicitacao(array $params)
    {
        try {
            self::validaImportacaoSolicitacao($params);
            /**
             * Itemsoli
             *
             * @var $repository \Geral\Repositories\Itemsoli
             */
            $repository = \Geral\Entities\Itemsoli::getRepository();
            return $repository->importaSolicitacao($params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Insere dados na tabela de MOVSOLEXT apos a inclusao de uma solicitacao, caso esta seja marcada como extra.
     *
     * @param array $linha Linha
     *
     * @return null
     * @throws \Exception
     */
    public static function insereMovsolext($linha)
    {
        $movsol = array();
        if ($linha['IDSOLEXTRA'] === 'S') {
            $movsol["DSJUSTSOLEXT"] = $linha['DSJUSTSOLEXT'];
            $movsol["CDFILSOLI"]    = $linha['CDFILSOLI'];
            $movsol["NRSOLICMP"]    = tk_get_context_vars('NRSOLICMP', 'cmp03100');
            $movsol["DTSOLCMP"]     = $linha['DTSOLCMP'];
            $movsol["CDFILLANCA"]   = $linha['CDFILLANCA'];
            $movsol["NRORG"]        = \Geral\Util\Ambiente::getNrorg();
            $movsol["OPERADOR"]     = \Geral\Util\Ambiente::getCdOperador();

            if (DateUtil::dataPertenceAoFormato(
                $movsol["DTSOLCMP"], DateUtil::FORMATO_BRASILEIRO_DATAHORA
            )) {
                $movsol["DTSOLCMP"] = DateUtil::truncateString($movsol["DTSOLCMP"]);
            }

            try {
                $movimento = new \Geral\Entities\Movsolext();
                $movimento->setCdfillancext($movsol['CDFILLANCA']);
                $movimento->setCdfilsolext($movsol['CDFILSOLI']);
                $movimento->setCdoperador($movsol['OPERADOR']);
                $movimento->setDsjustsolext($movsol['DSJUSTSOLEXT']);
                $movimento->setDtmovsolext(
                    DateUtil::getDataDeString($movsol["DTSOLCMP"])
                );
                $movimento->setNrsolimovext($movsol['NRSOLICMP']);
                $movimento->setNrorg($movsol['NRORG']);
                $movimento->setDtinclusao(
                    DateUtil::getDataDeString(
                        DateUtil::getDataAtual()->format('d/m/Y')
                    )
                );
                $movimento->setCdoperinclusao($movsol['OPERADOR']);
                $movimento->persist();

                \Setforms2\Shared\Registry::getEntityManager()->flush();
            } catch (\Exception $e) {
                throw $e;
            }
        }

    }

    /**
     * Remove todos os itens da solicitacao entre uma faixa de produtos.
     *
     * @param String $nrorg      Nrorg
     * @param String $cdfilsoli  Cdfilsoli
     * @param String $cdfillanca Cdfillanca
     * @param String $nrsolicmp  Nrsolicmp
     * @param String $cdprodini  Cdprodini
     * @param String $cdprodfin  Cdprodfin
     * @param String $itens_marcados
     *
     * @return null
     * @throws \Exception
     */
    public static function deletaListaProdutos(
        $nrorg, $cdfilsoli, $cdfillanca, $nrsolicmp, $cdprodini, $cdprodfin, $itens_marcados
    )
    {
        //Desabilita trigger usada na tabela ITEMSOLI
        Movopsol::disableTrigger();
        try {
            self::validaDeletarListaProdutos($cdprodini, $cdprodfin);
            self::verificaUnicoProdutoSolicitacao(
                $nrorg, $cdfilsoli, $cdfillanca, $nrsolicmp
            );
            /**
             * Itemsoli
             *
             * @var $repository \Geral\Repositories\Itemsoli
             */
            $repository = \Geral\Entities\Itemsoli::getRepository();
//            $repository->deletaListaProdutos(
//                $nrorg, $cdfillanca, $cdfilsoli, $nrsolicmp, $cdprodini,
//                $cdprodfin
//            );
            $repository->deletaItemsoli(
                $cdfilsoli, $cdfillanca, $nrsolicmp, $nrorg, $itens_marcados
            );
        } catch (\Exception $e) {
            throw $e;
        }
        //Habilita trigger usada na tabela ITEMSOLI
        Movopsol::enableTrigger();
    }


    /**
     * Salva itens de solicitacao
     *
     * @param String $cdproduto     Cdproduto
     * @param String $dtprogentr    Dtprogentr
     * @param String $qtsoliprod    Qtsoliprod
     * @param String $cdfilsoli     Cdfilsoli
     * @param String $cdfillanca    Cdfillanca
     * @param String $nrsolicmp     Nrsolicmp
     * @param String $nrorg         Nrorg
     * @param String $justificativa Justificativa
     * @param String $observacao    Observacao
     * @param String $cdoperador    Cdoperador
     * @param String $dtatual       Dtatual
     * @param String $idaltplalog   Idaltplalog
     *
     * @return null
     * @throws \Exception
     */
    public static function salvaItemSolicitacao(
        $cdproduto, $dtprogentr, $qtsoliprod, $cdfilsoli, $cdfillanca,
        $nrsolicmp, $nrorg, $justificativa, $observacao, $cdoperador, $dtatual,
        $idaltplalog, $primeira_vez
    )
    {
        //Desabilita trigger usada na tabela ITEMSOLI
        Movopsol::disableTrigger();
        $idoperamovs = "A";
        $qtantoper = "0";
        $where = array();

        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();
            self::validaSalvarItemSolicitacao(
                $cdproduto, $dtprogentr, $qtsoliprod, $justificativa, $cdfilsoli, $primeira_vez
            );
            $idsoliaprov = self::verificaProdutoAprovadoCompra(
                $cdfilsoli, $cdfillanca, $nrsolicmp, $cdproduto, $dtprogentr,
                $nrorg, $primeira_vez
            );
            $where = array(
                $cdfilsoli,
                $cdfillanca,
                $nrsolicmp,
                $cdproduto,
                $dtprogentr,
                $nrorg,
            );

            /** @var \Geral\Entities\Itemsoli $itemsoli */
            $itemsoli = \Geral\Entities\Itemsoli::getObjectByWhere(
                'CDFILSOLI=? AND CDFILLAN=? AND NRSOLICMP=? AND
                CDPRODUTO=? AND DTPROGENTR=? AND NRORG=?',
                $where
            );

            if (($itemsoli instanceof \Geral\Entities\Itemsoli) === false) {
                $retorno = self::validaDataProgramadaEntrega($dtprogentr);
                if ($retorno === false) {
                    \Compras\Logic\SolicitacaoCompras\Exception\CamposNaoPreenchidos::mensagemDataProcessamento();
                }
                $itemsoli = new \Geral\Entities\Itemsoli();
                $itemsoli->setCdfilsoli($cdfilsoli);
                $itemsoli->setCdfillan($cdfillanca);
                $itemsoli->setNrsolicmp($nrsolicmp);
                $itemsoli->setCdproduto($cdproduto);
                $itemsoli->setDtprogentr(
                    DateUtil::getDataDeString(
                        $dtprogentr, DateUtil::FORMATO_BRASILEIRO, true
                    )
                );
                $itemsoli->setNrorg($nrorg);
                $itemsoli->setNrorginclusao($nrorg);
                $itemsoli->setDtinclusao(DateUtil::getDataAtual());
                $itemsoli->setCdoperinclusao($cdoperador);
                $itemsoli->setIditmsolextra("N");
                $itemsoli->setIdsoliaprov($idsoliaprov);
                $idoperamovs = "I";
                $qtantoper = "0";
                $itemsoli->setQtsoliprod($qtsoliprod);
                $itemsoli->setDtultatu(DateUtil::getDataAtual());
                $itemsoli->setCdoperultatu($cdoperador);
                $itemsoli->setNrorgultatu($nrorg);
                $itemsoli->setDtcmpanterior(DateUtil::getDataAtual());
                $itemsoli->setTxobsitmsol($observacao);
                $itemsoli->persist();

                $connection->flush();
                $connection->commit();

            } else {
                $qtantoper = $itemsoli->getQtsoliprod();
                $itemsoli->setQtsoliprod($qtsoliprod);
                $itemsoli->setDtultatu(DateUtil::getDataAtual());
                $itemsoli->setCdoperultatu($cdoperador);
                $itemsoli->setNrorgultatu(\Geral\Util\Ambiente::getNrorg());
                $itemsoli->setDtcmpanterior(DateUtil::getDataAtual());
                $itemsoli->setTxobsitmsol($observacao);
                $itemsoli->persist();

                $connection->flush();
                $connection->commit();

            }

            /** @todo Anteriormente salva justificativa de alteração apenas quando a unidade exigia justificativa */
            if ($primeira_vez != 'S') {
//                $insere_justificativa = Movopsol::verificaFilialInsereJust(
//                    $cdfilsoli
//                );
//                if (!empty($insere_justificativa)) {
//                    if ($insere_justificativa[0]['IDEXIGAPRGERCMP'] == 'S') {
                        //Insere justificativa nos itens de solicitacao na tabela MOVOPSOL
                if (!empty($justificativa)) {
                        Movopsol::insereJustAlteracao(
                            $cdfilsoli, $cdfillanca, $nrsolicmp, $cdproduto, $dtprogentr,
                            $dtatual, $cdoperador, $justificativa, $qtantoper, $qtsoliprod,
                            $idaltplalog, $idoperamovs, $nrorg
                        );
                }
//                    }
//                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
        //Habilita trigger usada na tabela ITEMSOLI
        Movopsol::enableTrigger();
    }


    /**
     * Remove todos os produtos de uma solicitacao, dentro de um intervalo de datas.
     *
     * @param String $cdfilsoli  Cdfilsoli
     * @param String $cdfillanca Cdfillanca
     * @param String $nrsolicmp  Nrsolicmp
     * @param String $nrorg      Nrorg
     * @param String $dtinicial  Dtinicial
     * @param String $dtfinal    Dtfinal
     * @param String  $itens_marcados
     *
     * @return null
     * @throws \Exception
     */
    public static function deletaProdutosPorIntervaloDeDatas(
        $cdfilsoli, $cdfillanca, $nrsolicmp, $nrorg, $dtinicial, $dtfinal, $itens_marcados
    )
    {
        try {
            self::validaDeletarPorIntervaloDeDatas($dtinicial, $dtfinal);
            self::verificaUnicoProdutoSolicitacao(
                $nrorg, $cdfilsoli, $cdfillanca, $nrsolicmp
            );
            /**
             * Itemsoli
             *
             * @var $repository \Geral\Repositories\Itemsoli
             */
            $repository = \Geral\Entities\Itemsoli::getRepository();
            $repository->deletarProdutosPorIntervaloDeDatas(
                $cdfilsoli, $cdfillanca, $nrsolicmp, $nrorg, $dtinicial,
                $dtfinal
            );

//            $repository->deletaItemsoliEntreDatas(
//                $cdfilsoli, $cdfillanca, $nrsolicmp, $nrorg, $itens_marcados, $dtinicial, $dtfinal
//            );
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Altera o status de aprovacao dos itens de uma solicitacao, segundo os parametros passados. Este metodo permite
     * que a alteracao de status seja feita a nivel de solicitacao(todos os itens de uma determinada solicitacao terao
     * seu status alterado para o mesmo valor), caso os parametros de cdproduto e dtprogentr nao sejam informados, ou a
     * nivel de item, passando todos os parametros.
     *
     * @param String $idsoliaprov Idsoliaprov
     * @param String $cdfilsoli   Cdfilsoli
     * @param String $cdfillan    Cdfillan
     * @param String $nrsolicmp   Nrsolicmp
     * @param String $nrorg       Nrorg
     * @param String $cdproduto   Cdproduto
     * @param String $dtprogentr  Dtprogentr
     *
     * @return null
     * @throws \Exception
     */
    public static function alterarStatusItensSolicitacao(
        $idsoliaprov, $cdfilsoli, $cdfillan, $nrsolicmp, $nrorg,
        $cdproduto = null, $dtprogentr = null
    )
    {
        $where
            = "CDFILSOLI = ? AND CDFILLAN = ? AND NRSOLICMP = ? AND NRORG = ?";
        $params = array(
            $cdfilsoli,
            $cdfillan,
            $nrsolicmp,
            $nrorg,
        );

        if ($cdproduto) {
            $where .= " AND CDPRODUTO = ? AND DTPROGENTR = ?";
            $params[] = $cdproduto;
            $params[] = $dtprogentr->format(DateUtil::FORMATO_BRASILEIRO);
        }

        try {
            $itens = \Geral\Entities\Itemsoli::getObjectsByWhere(
                $where, $params
            );

            /**
             * Itemsoli
             *
             * @var $itemsoli \Geral\Entities\Itemsoli
             */
            foreach ($itens as $itemsoli) {
                $itemsoli->setIdsoliaprov($idsoliaprov);
                $itemsoli->persist();
            }

            \Setforms2\Shared\Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Verifica todas as condicoes a ser cumpridas antes da gravacao da lista de produtos.
     *
     * @param String $params Params
     *
     * @return null
     */
    private static function validaGravacaoListaProdutos($params)
    {
        if ((bool)$params["NRSOLICMP"] === false) {
            \Compras\Logic\SolicitacaoCompras\Exception\CamposNaoPreenchidos::solicitacaoNaoSalva(
            );
        }
        // Caso algum destes dados nao existirem, o sistema deve informar que todos estes campos sao obrigatorios.
        if ((bool)$params["DTPROGENTR"] === false) {
            \Compras\Logic\SolicitacaoCompras\Exception\CamposNaoPreenchidos::mensagemDataEntrega(
            );
        }
        if ((bool)$params["CDPRODINI"] === false) {
            \Compras\Logic\SolicitacaoCompras\Exception\CamposNaoPreenchidos::mensagemProdutoInicial(
            );
        }
        if ((bool)$params["CDPRODFIN"] === false) {
            \Compras\Logic\SolicitacaoCompras\Exception\CamposNaoPreenchidos::mensagemProdutoFinal(
            );
        }
    }

    /**
     * Verifica se todas as condicoes para a importacao foram cumpridas.
     *
     * @param String $params Params
     *
     * @return null
     */
    private static function validaImportacaoSolicitacao($params)
    {
        if ((bool)$params["NRSOLICMP"] === false) {
            \Compras\Logic\SolicitacaoCompras\Exception\CamposNaoPreenchidos::solicitacaoNaoSalva(
            );
        }

        if ((bool)$params['CDFILSOLIMP'] === false
            || (bool)$params['NRSOLICMPIMP'] === false
            || (bool)$params['DTPROGENTR'] === false
        ) {
            \Compras\Logic\SolicitacaoCompras\Exception\CamposNaoPreenchidos::importacaoSolicitacao(
            );
        }
    }

    /**
     * Verifica se todas as condicoes para a confirmacao de itens na solicitacao estao corretas.
     *
     * @param String $cdproduto     Cdproduto
     * @param String $dtprogentr    Dtprogentr
     * @param String $qtsoliprod    Qtsoliprod
     * @param String $justificativa Justificativa
     * @param String $cdfilsoli Cdfilsoli
     *
     * @return null
     */
    private static function validaSalvarItemSolicitacao(
        $cdproduto, $dtprogentr, $qtsoliprod, $justificativa, $cdfilsoli,
        $primeira_vez
    )
    {
        $validaQntd = self::validaQuantidade($qtsoliprod);
        if ((bool)$cdproduto === false || (bool)$dtprogentr === false
            || (bool)$qtsoliprod === false
        ) {
            \Compras\Logic\SolicitacaoCompras\Exception\CamposNaoPreenchidos::mensagemPadrao(
            );
        } elseif ($validaQntd == false) {
            \Compras\Logic\SolicitacaoCompras\Exception\CamposNaoPreenchidos::mensagemQuantidadeMaiorZero(
            );
        } else {
            if ($primeira_vez != 'S') {
                $insere_justificativa = Movopsol::verificaFilialInsereJust(
                    $cdfilsoli
                );
                if (!empty($insere_justificativa)) {
                    if ($insere_justificativa[0]['IDEXIGAPRGERCMP'] == 'S') {
                        if (empty($justificativa)) {
                            \Compras\Logic\SolicitacaoCompras\Exception\CamposNaoPreenchidos::mensagemJustificativaAlteracao(
                            );
                        }
                    }
                }
            }
        }

    }

    /**
     * Verifica se os campos de produto inicial e produto final estao preenchidos. Caso contrario, nao sera permitido
     * executar o processo de exclusao automatica.
     *
     * @param String $cdprodini Cdprodini
     * @param String $cdprodfin Cdprodfin
     *
     * @return null
     */
    private static function validaDeletarListaProdutos($cdprodini, $cdprodfin)
    {
        if ((bool)$cdprodfin === false) {
            $cdprodini = $cdprodfin;
        }

        if ((bool)$cdprodini === false || (bool)$cdprodfin === false) {
            \Compras\Logic\SolicitacaoCompras\Exception\CamposNaoPreenchidos::mensagemPadrao(
            );
        }
    }

    /**
     * Verifica se todas as condicoes necessarias para a exclusao automatica por intervalo de datas foram preenchidas.
     *
     * @param String $dtinicial Dtinicial
     * @param String $dtfinal   Dtfinal
     *
     * @return null
     * @throws \Geral\Exceptions\IntervaloIrregularDeDatas
     */
    private static function validaDeletarPorIntervaloDeDatas(
        $dtinicial, $dtfinal
    )
    {
        $dtinicial = DateUtil::getDataDeString($dtinicial);
        $dtfinal = DateUtil::getDataDeString($dtfinal);

        if ((bool)$dtfinal === false) {
            $dtfinal = $dtinicial;
        }

        if ((bool)$dtinicial === false || (bool)$dtfinal === false) {
            \Compras\Logic\SolicitacaoCompras\Exception\CamposNaoPreenchidos::mensagemPadrao(
            );
        }

        // Verifica se a data inicial esta antes da data final.
        if (DateUtil::comparaDatas($dtfinal, $dtinicial)) {
            throw new \Geral\Exceptions\IntervaloIrregularDeDatas();
        }

    }

    /**
    * Retorna o custo do produto

     * @param String $cdfilsoli Cdfilsoli
     * @param String $cdproduto Cdproduto
     * @param String $dtentrega Dtentrega
     * @param String $nrorg     Nrorg
     *
     * @return null
     * @throws \Exception
     */
    public static function retornaCustoProduto(
        $cdfilsoli, $cdproduto, $dtentrega, $nrorg
    )
    {
        /**
         * Custocmpprod
         *
         * @var $repository \Estoque\Repositories\Custocmpprod
         */
        try{
            $repository = \Estoque\Entities\Custocmpprod::getRepository();
            $custoProd = $repository->consultaFuncaoCustoProduto(
                $cdfilsoli, $cdproduto, $dtentrega, '06'
            );
//            $produto = \Geral\Entities\Produto::findOneByCdprodutoAndNrorg(
//                $cdproduto, $nrorg
//            );
            $produto = \Geral\Entities\Produto::buscaRegistro($cdproduto, $nrorg);
            $vrFatConv = ($produto instanceof \Geral\Entities\Produto)
                ? $produto->getVrfatoconv() : 1;
            return (float)$custoProd[0]["CUSTO"] * (float)$vrFatConv;
        } catch(\Exception $e){
            throw $e;
        }

    }

    /**
     * Lanca excecao se a solicitacao possuir apenas um item.
     *
     * @param String $nrorg      Nrorg
     * @param String $cdfilsoli  Cdfilsoli
     * @param String $cdfillanca Cdfillanca
     * @param String $nrsolicmp  Nrsolicmp
     *
     * @return null
     */
    public static function verificaUnicoProdutoSolicitacao(
        $nrorg, $cdfilsoli, $cdfillanca, $nrsolicmp
    )
    {
        $ItemSoli = \Geral\Entities\Itemsoli::getObjectsByWhere(
            "NRORG = ? AND CDFILSOLI = ? AND CDFILLAN = ? AND NRSOLICMP = ?",
            array($nrorg, $cdfilsoli, $cdfillanca, $nrsolicmp)
        );

        if (count($ItemSoli) == 1) {
            \Compras\Logic\SolicitacaoCompras\Exception\ProdutoSoli::excluiUltimoItemSoli(
            );
        }

    }

    /**
     * Busca central de compras da filial
     *
     * @param String $nrorg    Nrorg
     * @param String $cdfilial Cdfilial
     *
     * @return mixed
     */
    public static function buscaCentralCompras($nrorg, $cdfilial)
    {
        $array = array($nrorg, $cdfilial);
        $object = \Geral\Entities\Filial::getObjectByWhere(
            'NRORG=? AND CDFILIAL=?', $array
        );
        /**
         * Filial
         *
         * @var $object \Geral\Entities\Filial
         */
        $Cdfilcentcmp = $object->getCdfilcentcmp();

        if ($Cdfilcentcmp) {
            return $Cdfilcentcmp;
        } else {
            return $cdfilial;
        }
    }

    /**
     * Consulta a tabela Itemsoli verificando quantos registros determinada solicitacao possui,
     * atualizando o total na tabela Solicita
     *
     * @param String $cdfilsoli Cdfilsoli
     * @param String $cdfillan  Cdfillan
     * @param String $nrsolicmp Nrsolicmp
     * @param String $nrorg     Nrorg
     *
     * @return null
     * @throws \Exception
     */
    public static function atualizaQtitemsolic(
        $cdfilsoli, $cdfillan, $nrsolicmp, $nrorg
    )
    {
        try {
            /**
             * Itemsoli
             *
             * @var $itensSolicitacao \Geral\Entities\Itemsoli
             */
            $itensSolicitacao = \Geral\Entities\Itemsoli::getObjectsByWhere(
                'CDFILSOLI = ? AND CDFILLAN = ? AND NRSOLICMP = ?',
                array($cdfilsoli, $cdfillan, $nrsolicmp)
            );
            $qtitemsolic = count($itensSolicitacao);

            /**
             * Solicita
             *
             * @var $solicita \Geral\Entities\Solicita
             */
            $solicita = \Geral\Entities\Solicita::getObjectByWhere(
                'CDFILSOLI = ? AND CDFILLANCA = ? AND NRSOLICMP = ? AND NRORG = ?',
                array($cdfilsoli, $cdfillan, $nrsolicmp, $nrorg)
            );
            $solicita->setQtitemsolic($qtitemsolic);
            $solicita->persist();

            \Setforms2\Shared\Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Verifica se a quantidade do produto do item de solicita��o � maior que zero.
     *
     * @param String $quantidade Quantidade
     *
     * @return bool
     */
    public static function validaQuantidade($quantidade)
    {
        if ($quantidade >= 0) {
            return true;
        } else {
            return false;
        }

    }

    /**
     * Método valida o parametro 'Utiliza Aprovação Solicitação de Compras'
     * Verificando se é possível incluir um novo item
     * Se é permitido alteração na quantidade
     * E retornando se o produto deve ser incluido com o status de aprovado.
     *
     * @param $cdfilsoli
     * @param $cdfillan
     * @param $nrsolicmp
     * @param $cdproduto
     * @param $dtprogentr
     * @param $nrorg
     *
     * @return string
     */
    public static function verificaProdutoAprovadoCompra(
        $cdfilsoli, $cdfillan, $nrsolicmp, $cdproduto, $dtprogentr, $nrorg, $primeira_vez
    )
    {
        $prodAprovado = false;
        $prodExiste = false;
        $prodExisteAprovado = false;
        tk_set_env_var('IMPEDIR_INCLUSAO', false);

        /** @var  $parcmpsup \Geral\Entities\Parcmpsup */
        $parcmpsup = \Geral\Entities\Parcmpsup::getObjectByWhere(
            'CDFILIAL=?',
            array($cdfilsoli)
        );

        if ($parcmpsup->getIdutaprovsol() == 'S') {
            $idperaltsapr = $parcmpsup->getIdperaltsapr();
            $incluirProdAprov = $idperaltsapr[0];
            $permiteAlterQtde = $idperaltsapr[1];
            $impedirInclusao = $parcmpsup->getIdimpincsolapr();

            $itemsoli = \Geral\Entities\Itemsoli::getObjectsByWhere(
                'CDFILSOLI = ? AND CDFILLAN = ? AND NRSOLICMP = ? AND NRORG = ?',
                array($cdfilsoli, $cdfillan, $nrsolicmp, $nrorg)
            );

            foreach ($itemsoli as $item) {
                $idsoliaprov = $item->getIdsoliaprov();
                $produto = $item->getCdproduto();
                $dataEntrega = $item->getDtprogentr()->format('d/m/Y');
                if ($idsoliaprov == 'S') {
                    $prodAprovado = true;
                    if ($produto == $cdproduto && $dataEntrega == $dtprogentr) {
                        $prodExisteAprovado = true;
                    }
                } else {
                    if ($produto == $cdproduto && $dataEntrega == $dtprogentr) {
                        $prodExiste = true;
                    }
                }
            }

            if ($permiteAlterQtde == 'N' && $prodExisteAprovado == true) {
                \Compras\Logic\SolicitacaoCompras\Exception\CamposNaoPreenchidos::mensagemProdExisteAprovado(
                );
            } else {
                if ($impedirInclusao == 'S' && $prodAprovado == true
                    && $prodExiste == false && $primeira_vez != 'S'
                ) {
                    tk_set_env_var('IMPEDIR_INCLUSAO', true);
                    \Compras\Logic\SolicitacaoCompras\Exception\CamposNaoPreenchidos::mensagemItemAprovado();
                }
            }

            if ($incluirProdAprov == 'S') {
                return 'S';
            } else {
                return 'N';
            }
        }
    }


    /**
     * Verifica se a data programada de entrega informada pelo usuário é menor que a data de processamento.
     * Caso seja menor, retorna false
     * Caso seja igual ou maior, retorna true
     *
     * @param String $dtprogentr Dtprogentr
     *
     * @return bool
     */
    public static function validaDataProgramadaEntrega($dtprogentr)
    {
        $dtProcessamento = \Estoque\Util\Parametros::getDataProcessamento();
        if ($dtprogentr != $dtProcessamento) {
            $menorData = DateUtil::retornaMenorData(
                $dtprogentr, $dtProcessamento
            );
            if ($menorData == $dtprogentr) {
                return false;
            } else {
                return true;
            }
        } else {
            return true;
        }
    }

    /**
     * Exclui solicitacao
     *
     * @static
     *
     * @param $nrorg
     * @param $cdfillanca
     * @param $cdfilsoli
     * @param $nrsolicmp
     *
     * @return bool
     */
    public static function excluiSolicitacao(
        $nrorg, $cdfillanca, $cdfilsoli, $nrsolicmp
    )
    {
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();
        try {
            /** @var Solicita $obj_Solicita */
            $obj_Solicita = Solicita::getObjectByWhere(
                'NRORG = ? AND CDFILLANCA = ? AND CDFILSOLI = ? AND NRSOLICMP = ?',
                array($nrorg, $cdfillanca, $cdfilsoli, $nrsolicmp)
            );

            if ($obj_Solicita instanceof \Geral\Entities\Solicita) {
                $obj_Solicita->delete();

                $connection->flush();
                $connection->commit();

                return true;
            }
            return false;
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
            return false;
        }
    }

    /**
     * Exclui itens da solicitacao
     *
     * @static
     *
     * @param $nrorg
     * @param $cdfillanc
     * @param $cdfilsoli
     * @param $nrsolicmp
     *
     * @return null
     */
    public static function excluiItemSolicitacao(
        $nrorg, $cdfilsoli, $cdfillan, $nrsolicmp
    )
    {
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();

        try {
            $array_Itemsoli = Itemsoli::getObjectsByWhere(
                'NRORG = ? AND CDFILLAN = ? AND CDFILSOLI = ? AND NRSOLICMP = ?',
                array($nrorg, $cdfillan, $cdfilsoli, $nrsolicmp)
            );

            if (!empty($array_Itemsoli)) {
                /** @var Itemsoli $obj_Itemsoli */
                foreach ($array_Itemsoli as $chave => $obj_Itemsoli) {
                    if ($obj_Itemsoli instanceof \Geral\Entities\Itemsoli) {
                        $connection->beginTransaction();

                        $obj_Itemsoli->delete();

                        $connection->flush();
                        $connection->commit();
                    }
                }
            }
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
            return false;
        }
    }


    /**
     * Exclui justificativa de alteracao dos itens da solicitacao
     *
     * @static
     *
     * @param $nrorg
     * @param $cdfillanmov
     * @param $cdfilsolmov
     * @param $nrsolimov
     *
     * @return null
     */
    public static function excluiJustificativaAlteracao(
        $nrorg, $cdfillanmov, $cdfilsolmov, $nrsolimov
    )
    {
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();

        try {
            $array_Movopsol = Movopsol::verificaJust($nrorg, $cdfilsolmov, $cdfillanmov, $nrsolimov);

            if (!empty($array_Movopsol)) {
                Movopsol::excluiJust($nrorg, $cdfilsolmov, $cdfillanmov, $nrsolimov);

                $connection->flush();
                $connection->commit();
            }
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
            return false;
        }
    }


    /**
     * Valida Filial Solicitante
     * @static
     * @param $cdfilsoli
     * @param $nrorg
     * @return null
     */
    public static function validaFilialSolicitante($cdfilsoli, $nrorg)
    {
        /** @var  \Geral\Repositories\Solicita $repositorio_Solicita */
        $repositorio_Solicita = \Geral\Entities\Solicita::getRepository();
        $parametros = $repositorio_Solicita->verificaParametrosFilialSolicitante($cdfilsoli, $nrorg);

        if (empty($parametros)) {
            tk_abort(tk_get_message_by_name('msg_param_compra_filial_inexistente'));
        }

        /** @var  \Geral\Entities\Filial $obj_Filial */
        $obj_Filial = \Geral\Entities\Filial::retornaFilial($cdfilsoli, $nrorg);

        if ($obj_Filial->getIdcntrresc() === 'S') {
            tk_abort(tk_get_message_by_name('msg_filial_contrato_rescindido'));
        }
        $cdregiao = $obj_Filial->getCdregiao();
        if (empty($cdregiao)) {
            tk_abort(tk_get_message_by_name('msg_filial_sem_regiao'));
        }
    }

}
